/**
 * Sergey Kopeliovich (burunduk30@gmail.com)
 *
 * Идея: 
 *		Разделяй и властвуй. Построили выпуклые оболочки половин, если искомая точка в одной из них, перешли в рекурсию.
 *		Иначе бинпоиск по триангуляции части общей выпуклой оболочки между двумя мелкими выпуклыми оболочками.
 *
 * Время: q*(n + ans*nlogn) (n -- общая касательная, ans*nlogn можно улучшить до ans*n)
 * Запросы: не более 3log_2n
 *
 * P.S. Получено из igeom_sk_tangent_n2.cpp заменой общей касательной на линейную
 * 
 * Баллы: 100
 */

#include <bits/stdc++.h>

using namespace std;

#define err(...) fprintf(stderr, __VA_ARGS__), fflush(stderr)
#define forn(i, n) for (int i = 0; i < (int)(n); i++)
#define all(a) (a).begin(), (a).end()

typedef long long ll;
typedef pair<int, int> pii;

struct pnt {
	int x, y, i;
	bool operator < ( pnt p ) const { return pii(x, y) < pii(p.x, p.y); }
	pnt operator - ( pnt p ) { return {x - p.x, y - p.y}; }
	ll operator * ( pnt p ) { return (ll)x * p.y - (ll)y * p.x; }
};

int main() {
	int n;
	assert(scanf("%d", &n) == 1);
	//err("polygon: %d points\n", n);
	pnt p[n];
	forn(i, n)
		assert(scanf("%d%d", &p[i].x, &p[i].y) == 2), p[i].i = i;
	sort(p, p + n);

	auto out = [&]( char c, const vector<int> &ind ) {
		printf("%c %ld", c, ind.size());
		for (int i : ind)
			printf(" %d", p[i].i + 1);
		puts("");
		fflush(stdout);
	};
	auto ask = [&]( const vector<int> &ind ) {
		out('?', ind);
		char s[9];
		assert(scanf("%s", s) == 1);
		return tolower(s[0]) == 'y';
	};
	auto convexHull = [&]( int l, int r ) { // [l, r)
		vector<int> ind, ans;
		for (; l < r; l++)
			ind.push_back(l);
		auto pless = [&]( pnt a, pnt b ) { return a * b > 0; };
		auto iless = [&]( int i, int j ) { return pless(p[i] - p[ind[0]], p[j] - p[ind[0]]); };
		sort(ind.begin() + 1, ind.end(), iless);
		for (int i : ind) {
			while (ans.size() >= 2U && !pless(p[ans.back()] - p[ans[ans.size() - 2]], p[i] - p[ans.back()]))
				ans.pop_back();
			ans.push_back(i);
		}
		return ans;
	};
	auto area = [&]( int i, int j, int k ) {
		return abs((p[i] - p[j]) * (p[j] - p[k]));
	};
	auto is_in = [&]( int p, int a, int b, int c ) {
		return area(a, b, c) == area(a, b, p) + area(a, c, p) + area(c, b, p);
	};
	function<void(int, int)> solve = [&]( int l, int r ) { // [l, r)
		int ri = -1, rj = -1, rk = -1;
		ll opt = LLONG_MAX, tmp;
		auto relax = [&]( int i, int j, int k ) {
			if (ask({i, j, k}) && (tmp = area(i, j, k)) < opt)
				opt = tmp, ri = i, rj = j, rk = k;
		};
		if (r - l < 6) { // can not split into two parts => solve in O((r-l)^3)
			for (int i = l; i < r; i++)
				for (int j = l; j < i; j++)
					for (int k = l; k < j; k++)
						relax(i, j, k);
			out('!', {ri, rj, rk});
			return;
		}
		int m = (l + r) / 2;
		vector<int> cl = convexHull(l, m);
		vector<int> cr = convexHull(m, r);
		if (ask(cl))
			solve(l, m);
		else if (ask(cr))
			solve(m, r);
		else {
			auto find_tangent = [&]( int orient, int &l, int &r ) {
				int sl = cl.size(), sr = cr.size();
				auto fail = [&]( int i ) {
					return (p[i] - p[cl[l]]) * (p[cr[r]] - p[cl[l]]) * orient > 0;
				};
				l = 0, r = 0;
				for (int good = 0; !good; ) {
					good = 1;
					for (int d = -1; d <= 1; d += 2) {
						int l1 = (l + sl + d) % sl;
						int r1 = (r + sr + d) % sr;
						if (fail(cl[l1])) good = 0, l = l1;
						if (fail(cr[r1])) good = 0, r = r1;
					}
				}
			};
			int cl_min, cr_min, cl_max, cr_max;
			find_tangent(1, cl_min, cr_min);
			find_tangent(-1, cl_max, cr_max);
			if (cl_min >= cl_max) cl_max += cl.size();
			if (cr_min <= cr_max) cr_min += cr.size();
			int li = cl_min, ri = cr_min;
			vector<pii> pairs(1, pii(li, ri));
			while (li != cl_max || ri != cr_max) {
				int l_next = li + 1, r_next = ri - 1;
				int l1 = cl[li % cl.size()], l2 = cl[l_next % cl.size()];
				int r1 = cr[ri % cr.size()], r2 = cr[r_next % cr.size()];
				if (ri == cr_max || (li != cl_max && !is_in(r2, l1, r1, l2) && (p[l2] - p[l1]) * (p[l1] - p[r1]) > 0))
					li = l_next;
				else
					ri = r_next;
				pairs.push_back(pii(li, ri));
			}
			int l = 0, r = pairs.size() - 1;
			auto check = [&]( int m ) {
				if (m <= 0) return false;
				int l0 = cl[cl_min % cl.size()], lm = cl[pairs[m].first % cl.size()];
				int r0 = cr[cr_min % cr.size()], rm = cr[pairs[m].second % cr.size()];
				if ((p[lm] - p[l0]) * (p[rm] - p[lm]) > 0)
					return (rm != r0 && ask({lm, rm, r0})) || (lm != l0 && ask({l0, r0, lm}));
				else
					return (lm != l0 && ask({lm, rm, l0})) || (rm != r0 && ask({l0, r0, rm}));
			};
			while (r - l > 1) {
				int m = (l + r) / 2;
				(check(m) ? r : l) = m;
			}
			// auto check_triangle = [&]( int r ) {
			// 	int l1 = pairs[r - 1].first, l2 = pairs[r].first;
			// 	int r1 = pairs[r - 1].second, r2 = pairs[r].second;
			// 	int third = (r1 != r2 ? cr[r1 % cr.size()] : cl[l1 % cl.size()]);
			// 	int a = cl[l2 % cl.size()], b = cr[r2 % cr.size()];
			// 	return ask({a, b, third});
			// };
			assert(check(r) && !check(r - 1));
			int l1 = pairs[r - 1].first, l2 = pairs[r].first;
			int r1 = pairs[r - 1].second, r2 = pairs[r].second;
			int third = (r1 != r2 ? cr[r1 % cr.size()] : cl[l1 % cl.size()]);
			int a = cl[l2 % cl.size()], b = cr[r2 % cr.size()];
			out('!', {a, b, third});
			return;
		}
	};

	int q;
	assert(scanf("%d", &q) == 1);
	//err("process %d queries\n", q);
	forn(i, q) {
		//err("! %d of %d\n", i, q);
		solve(0, n);
	}
}
