/**
 * Sergey Kopeliovich (burunduk30@gmail.com)
 *
 * Идея: 
 *		Разделяй и властвуй. Построили выпуклые оболочки половин, если искомая точка в одной из них, перешли в рекурсию.
 *		Иначе бинпоиск по триангуляции части общей выпуклой оболочки между двумя мелкими выпуклыми оболочками.
 *
 * Время: q*(n^2 + ans*nlogn) (n^2 -- общая касательная, можно соптимизить до n, ans*nlogn можно улучшить до ans*n)
 * Запросы: не более 3log_2n
 * 
 * Баллы: 80-90
 */

#include <bits/stdc++.h>

using namespace std;

#define err(...) fprintf(stderr, __VA_ARGS__), fflush(stderr)
#define forn(i, n) for (int i = 0; i < (int)(n); i++)
#define all(a) (a).begin(), (a).end()

typedef long long ll;
typedef pair<int, int> pii;

struct pnt {
	int x, y, i;
	bool operator < ( pnt p ) const { return pii(x, y) < pii(p.x, p.y); }
	pnt operator - ( pnt p ) { return {x - p.x, y - p.y}; }
	ll operator * ( pnt p ) { return (ll)x * p.y - (ll)y * p.x; }
};

FILE *fpic = fopen("a.tex", "wt");
int X0 = -100, Y0 = -100, scale = 10;
void circle( const char *color, int x, int y ) {
	x -= X0, y -= Y0;
	x /= scale, y /= scale;
	fprintf(fpic, "\\fill [%s] (%d,%d) circle (15mm);\n", color, x, y);
}
void line( const char *color, int x1, int y1, int x2, int y2, int ind ) {
	// err("draw line!\n");
	x1 -= X0, y1 -= Y0;
	x2 -= X0, y2 -= Y0;
	x1 /= scale, y1 /= scale;
	x2 /= scale, y2 /= scale;
	fprintf(fpic, "\\draw [line width=0.01mm,%s] (%d,%d) -- (%d,%d);\n", color, x1, y1, x2, y2);
	// fprintf(fpic, "\\draw [line width=0.01mm,%s] (%d,%d) -- (%d,%d)  node [midway, above] () {$%d$};\n", color, x1, y1, x2, y2, ind);
}

int main() {
	int n;
	assert(scanf("%d", &n) == 1);
	//err("polygon: %d points\n", n);
	pnt p[n];
	forn(i, n)
		assert(scanf("%d%d", &p[i].x, &p[i].y) == 2), p[i].i = i;
	sort(p, p + n);
	auto out = [&]( char c, const vector<int> &ind ) {
		printf("%c %ld", c, ind.size());
		for (int i : ind)
			printf(" %d", p[i].i + 1);//, err(" %d", p[i].i + 1);
		// err("\n");
		puts("");
		fflush(stdout);
	};
	auto ask = [&]( const vector<int> &ind ) {
		out('?', ind);
		char s[9];
		assert(scanf("%s", s) == 1);
		//err("ask: answer is %d\n", tolower(s[0]) == 'y');
		return tolower(s[0]) == 'y';
	};
	auto convexHull = [&]( int l, int r ) { // [l, r)
		vector<int> ind, ans;
		for (; l < r; l++)
			ind.push_back(l);
		forn(i, ind.size() - 1)
			assert(p[ind[i]].x <= p[ind[i + 1]].x);
		auto pless = [&]( pnt a, pnt b ) { return a * b > 0; };
		auto iless = [&]( int i, int j ) { return pless(p[i] - p[ind[0]], p[j] - p[ind[0]]); };
		sort(ind.begin() + 1, ind.end(), iless);
		for (int i = 1; i + 1 < (int)ind.size(); i++) {
			assert(iless(ind[i], ind[i + 1]));
			assert(pless(p[ind[i]] - p[ind[0]], p[ind[i + 1]] - p[ind[0]]));
		}
		for (int i : ind) {
			while (ans.size() >= 2U && !pless(p[ans.back()] - p[ans[ans.size() - 2]], p[i] - p[ans.back()]))
				ans.pop_back();
			ans.push_back(i);
		}
		return ans;
	};
	auto area = [&]( int i, int j, int k ) {
		return abs((p[i] - p[j]) * (p[j] - p[k]));
	};
	auto is_in = [&]( int p, int a, int b, int c ) {
		return area(a, b, c) == area(a, b, p) + area(a, c, p) + area(c, b, p);
	};
	auto debug = [&]( int i ) {
		//err(" <%d,%d>", p[i].x, p[i].y);
	};
	function<void(int, int)> solve = [&]( int l, int r ) { // [l, r)
		//err("solve [%d..%d)\n", l, r);
		int ri = -1, rj = -1, rk = -1;
		ll opt = LLONG_MAX, tmp;
		auto relax = [&]( int i, int j, int k ) {
			if (ask({i, j, k}) && (tmp = area(i, j, k)) < opt)
				opt = tmp, ri = i, rj = j, rk = k;
		};
		if (r - l < 6) { // can not split into two parts => solve in O((r-l)^3)
			//err("case: small polygon\n");
			for (int i = l; i < r; i++)
				for (int j = l; j < i; j++)
					for (int k = l; k < j; k++)
						relax(i, j, k);
			out('!', {ri, rj, rk});
			return;
		}
		int m = (l + r) / 2;
		vector<int> cl = convexHull(l, m);
		vector<int> cr = convexHull(m, r);
		auto line = [&]( const char *color, int i, int j, int ind ) {
			::line(color, p[i].x, p[i].y, p[j].x, p[j].y, ind);
		};
		// err("hull sizes %ld %ld\n", cl.size(), cr.size());
		if (ask(cl)) {
			//err("call [%d..%d)\n", l, m);
			solve(l, m);
		}
		else if (ask(cr)) {
			//err("call [%d..%d)\n", m, r);
			solve(m, r);
		}
		else {
			//err("third case\n");
			for (int i : cl) circle("black", p[i].x, p[i].y);
			for (int i : cr) circle("black", p[i].x, p[i].y);
			circle("blue", p[cl[0]].x, p[cl[0]].y);
			circle("blue", p[cr[0]].x, p[cr[0]].y);
			forn(i, cl.size()) line("black", cl[i], cl[(i + 1) % cl.size()], i);
			forn(i, cr.size()) line("black", cr[i], cr[(i + 1) % cr.size()], i);
			auto find_tangent = [&]( int orient, int &l, int &r ) {
				int sl = cl.size(), sr = cr.size();
				for (l = 0; l < sl; l++)
					for (r = 0; r < sr; r++) {
						int a[] = {cl[(l + sl - 1) % sl], cl[(l + 1) % sl], cr[(r + sr - 1) % sr], cr[(r + 1) % sr]};
						bool good = 1;
						for (int i : a)
							good &= (p[i] - p[cl[l]]) * (p[cr[r]] - p[cl[l]]) * orient <= 0;
						if (good) {
							// err("tangent with orient=%d: %d %d (sizes: %d %d)\n", orient, l, r, sl, sr);
							return;
						}
					}
				assert(0);
			};
			int cl_min, cr_min, cl_max, cr_max;
			find_tangent(1, cl_min, cr_min);
			find_tangent(-1, cl_max, cr_max);
			if (cl_min >= cl_max) cl_max += cl.size();
			if (cr_min <= cr_max) cr_min += cr.size();
			// err("brute force hulls %d..%d %d..%d\n", cl_min, cl_max, cr_min, cr_max);

			// for (int i = l; i < r; i++) debug(i);
			// err(", cl =");
			// for (int i = cl_min; i <= cl_max; i++) debug(cl[i % cl.size()]);
			// err(", cr =");
			// for (int i = cr_max; i <= cr_min; i++) debug(cr[i % cr.size()]);
			// err("\n");
			int li = cl_min, ri = cr_min;
			vector<pii> pairs(1, pii(li, ri));
			int count = 0;
			// err("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
			while (li != cl_max || ri != cr_max) {
				// err("%d %d\n", li, ri);
				//assert(++count <= 100);
				int l_next = li + 1, r_next = ri - 1;
				int l1 = cl[li % cl.size()], l2 = cl[l_next % cl.size()];
				int r1 = cr[ri % cr.size()], r2 = cr[r_next % cr.size()];
				bool f = (p[l2] - p[l1]) * (p[l1] - p[r1]) > 0;
				//err(" [%d,%d,%d]", li != cl_max , !is_in(r2, l1, r1, l2) , f);
				if (ri == cr_max || (li != cl_max && !is_in(r2, l1, r1, l2) && f)) {
					li = l_next;
					//err("l");
				}
				else {
					ri = r_next;
					//err("r");
				}
				pairs.push_back(pii(li, ri));
			}
			//err("\n");
			forn(i, pairs.size()) {
				int l = cl[pairs[i].first % cl.size()];
				int r = cr[pairs[i].second % cr.size()];
				// err("pair[%d] = (%d,%d) = <%d,%d> and <%d,%d>\n", i, l, r, p[l].x, p[l].y, p[r].x, p[r].y);
				line("red", l, r, i);
			}
			// exit(0);
			// line("blue", cl[cl_min % cl.size()], cr[cr_min % cr.size()]);
			// line("green", cl[cl_max % cl.size()], cr[cr_max % cr.size()]);
			// int l = 0, r = pairs.size() - 1;
			// auto check = [&]( int m ) {
			// 	if (m <= 0) return false;
			// 	int l0 = cl[cl_min % cl.size()], lm = cl[pairs[m].first % cl.size()];
			// 	int r0 = cr[cr_min % cr.size()], rm = cr[pairs[m].second % cr.size()];
			// 	// err("l0=%d, r0=%d, lm=%d, rm=%d\n", l0, r0, lm, rm);
			// 	if (p[lm].y < p[rm].y)
			// 		return (rm != r0 && ask({lm, rm, r0})) || (lm != l0 && ask({l0, r0, lm}));
			// 	else
			// 		return (lm != l0 && ask({lm, rm, l0})) || (rm != r0 && ask({l0, r0, rm}));
			// };
			// while (r - l > 1) {
			// 	int m = (l + r) / 2;
			// 	bool f = check(m);
			// 	// err("m = %d --> f = %d\n", m, f);
			// 	(f ? r : l) = m;
			// }
			// // err("bs on [%d..%d] --> l=%d, r=%d\n", 0, (int)pairs.size() - 1, l, r);
			// assert(check(r));
			// assert(!check(r - 1));
			// // return;
			for (int r = 1; r < (int)pairs.size(); r++) {
				int l1 = pairs[r - 1].first, l2 = pairs[r].first;
				int r1 = pairs[r - 1].second, r2 = pairs[r].second;
				int third = (r1 != r2 ? cr[r1 % cr.size()] : cl[l1 % cl.size()]);
				int a = cl[l2 % cl.size()], b = cr[r2 % cr.size()];
				if (ask({a, b, third})) {
					out('!', {a, b, third});
					return;
				}
				// err("wow! answer : %d %d %d\n", cl[l2 % cl.size()], cr[r2 % cr.size()], third);
				// out('!', {cl[l2 % cl.size()], cr[r2 % cr.size()], third});
			}
		}
		//err("exit solve\n");
	};

	int q;
	assert(scanf("%d", &q) == 1);
	//err("process %d queries\n", q);
	forn(i, q) {
		//err("! %d of %d\n", i, q);
		solve(0, n);
	}
}
