/**
 * Sergey Kopeliovich (burunduk30@gmail.com)
 *
 * Идея: 
 *		Грехэмом по ходу построения выпуклой оболочки получили триангуляцию.
 *		Запрос: перебираем все треугольники из триангуляции.
 *
 * Время: nlogn + q*n
 * Запросы: <= n-3
 * 
 * Баллы: 60-70
 */

#include <bits/stdc++.h>

using namespace std;

#define err(...) fprintf(stderr, __VA_ARGS__), fflush(stderr)
#define forn(i, n) for (int i = 0; i < (int)(n); i++)
#define all(a) (a).begin(), (a).end()

typedef long long ll;
typedef pair<int, int> pii;

struct pnt {
	int x, y, i;
	bool operator < ( pnt p ) const { return pii(x, y) < pii(p.x, p.y); }
	pnt operator - ( pnt p ) { return {x - p.x, y - p.y}; }
	ll operator * ( pnt p ) { return (ll)x * p.y - (ll)y * p.x; }
};
vector<pnt> p;

struct triangle {
	int i, j, k;
};
vector<triangle> triangulation;

/** Tex.Drawing */
// FILE *fpic = fopen("a.tex", "wt");
// int min_x = 0, min_y = 0;
// double scale = 1;
// void cor( double &x, double &y ) {
// 	x -= min_x, y -= min_y;
// 	x /= scale, y /= scale;
// }
// void draw_circle( const char *color, double x, double y, int ind ) {
// 	cor(x, y);
// 	fprintf(fpic, "\\fill [%s] (%.3f,%.3f) circle (1mm) node () {$%d$};\n", color, x, y, ind);
// }
// void draw_triangle( const char *color, double x1, double y1, double x2, double y2, double x3, double y3, int ind ) {
// 	cor(x1, y1);
// 	cor(x2, y2);
// 	cor(x3, y3);
// 	fprintf(fpic, "\\draw [line width=0.01mm,%s] (%.3f,%.3f) -- (%.3f,%.3f) -- (%.3f,%.3f) -- (%.3f,%.3f) node [midway, above] () {$%d$};\n",
// 		color, x1, y1, x2, y2, x3, y3, x1, y1, ind);
// }
// void draw_triangle( const char *color, triangle t, int ind ) {
// 	draw_triangle(color, p[t.i].x, p[t.i].y, p[t.j].x, p[t.j].y, p[t.k].x, p[t.k].y, ind);
// }
/** END */

void convexHull() {
	vector<int> ind, ans;
	forn(i, p.size())
		ind.push_back(i);
	auto pless = [&]( pnt a, pnt b ) { return a * b > 0; };
	auto iless = [&]( int i, int j ) { return pless(p[i] - p[ind[0]], p[j] - p[ind[0]]); };
	sort(ind.begin() + 1, ind.end(), iless);
	for (int i = 1; i + 1 < (int)ind.size(); i++)
			triangulation.push_back({ind[0], ind[i], ind[i + 1]});
	for (int i : ind) {
		while (ans.size() >= 2U && !pless(p[ans.back()] - p[ans[ans.size() - 2]], p[i] - p[ans.back()])) {
			triangulation.push_back({ans.back(), ans[ans.size() - 2], i});
			ans.pop_back();
		}
		ans.push_back(i);
	}
}

void out( char c, triangle t ) {
	printf("%c 3 %d %d %d\n", c, p[t.i].i + 1, p[t.j].i + 1, p[t.k].i + 1);
	puts("");
	fflush(stdout);
};
bool ask( triangle t ) {
	out('?', t);
	char s[9];
	assert(scanf("%s", s) == 1);
	return tolower(s[0]) == 'y';
};


int main() {
	int n;
	assert(scanf("%d", &n) == 1);
	p.resize(n);

	forn(i, n) {
		assert(scanf("%d%d", &p[i].x, &p[i].y) == 2), p[i].i = i;
		// scale = max(scale, (double)max(abs(p[i].x), abs(p[i].y)));
		// min_x = min(min_x, p[i].x);
		// min_y = min(min_y, p[i].y);
	}
	// scale /= 6;
	sort(p.begin(), p.end());

	convexHull(); // build triangulation
	// forn(i, triangulation.size())
	// 	draw_triangle("red", triangulation[i], i);
	// forn(i, n)
	// 	draw_circle("blue", p[i].x, p[i].y, i);

	int q;
	assert(scanf("%d", &q) == 1);
	//err("process %d queries\n", q);
	forn(i, q) {
		//err("! %d of %d\n", i, q);
		bool good = 0;
		for (auto t : triangulation)
			if (ask(t)) {
				out('!', t);
				good = 1;
				break;
			}
		assert(good);
	}
}
