/**
 * Идея: 
 *  1. Строим выпуклую оболочку, бинпоиском находим в ней треугольник (1,i,i+1) с ответом Yes.
 *  2. Рекурсивно делим треугольник случайной точкой
 * Время: nlogn + ans*n
 * Запросы: ans <= logn + n, гипотеза: ans = O(logn)
 */

#include <bits/stdc++.h>

using namespace std;
vector<int> x, y;
long long area(int a, int b, int c)
{
	return 1LL * (x[b] - x[a]) * (y[c] - y[a]) - 1LL * (x[c] - x[a]) * (y[b] - y[a]);
}
bool cmp(int a, int b)
{
	return 1LL * x[a] * y[b] > 1LL * x[b] * y[a];
}
vector<int> convexHull(vector<int> a)
{
	int minPos = 0;
	for (int i = 0; i < (int)a.size(); i++)
	{
		if (make_pair(x[a[i]], y[a[i]]) < make_pair(x[a[minPos]], y[a[minPos]]))
		{
			minPos = i;
		}
	}
	if (minPos != 0)
	{
		swap(a[minPos], a[0]);
	}
	pair<int, int> mn = make_pair(x[a[0]], y[a[0]]);
	for (int i = 0; i < (int)a.size(); i++)
	{
		x[a[i]] -= mn.first;
		y[a[i]] -= mn.second;
	}
	sort(a.begin() + 1, a.end(), cmp);
	vector<int> st;
	for (int i = 0; i < (int)a.size(); i++)
	{
		while ((int)st.size() >= 2 && area(st[(int)st.size() - 2], st[(int)st.size() - 1], a[i]) <= 0) st.pop_back();
		st.push_back(a[i]);
	}
	for (int i = 0; i < (int)a.size(); i++)
	{
		x[a[i]] += mn.first;
		y[a[i]] += mn.second;
	}
	return st;
}
bool ask(int a, int b, int c)
{
	cout << "? 3 " << a << " " << b << " " << c << endl;
	string res;
	cin >> res;
	return res == "Yes";
}
bool inside(int a, int b, int c, int x)
{
	return abs(area(a, b, c)) == abs(area(a, b, x)) + abs(area(b, c, x)) + abs(area(a, c, x));
}
int main()
{
	//srand(time(NULL));
    srand(239); // we need stable results!
	int n;
	cin >> n;
    x.resize(n+1), y.resize(n+1);
	for (int i = 1; i <= n; i++)
	{
		cin >> x[i] >> y[i];
	}
	vector<int> tmp;
	for (int i = 1; i <= n; i++)
	{
		tmp.push_back(i);
	}
	vector<int> hull = convexHull(tmp);
    int q;
    cin >> q;
    //fprintf(stderr, "hull.size = %ld of %d\n", hull.size(), n);
    for (int qq = 1; qq <= q; qq++)
    {
    	int l = 1, r = (int)hull.size() - 1;
    	while (r - l > 1)
    	{
    		int m = (l + r) / 2;
    		cout << "? " << m + 1 << " ";
    		for (int i = 0; i <= m; i++)
    		{
    			cout << hull[i] << " ";
    		}
    		cout << endl;
    		string res;
    		cin >> res;
    		if (res == "Yes")
    		{
    			r = m;
    		}
    		else
    		{
    			l = m;
    		}
    	}
    	int a = hull[0], b = hull[r - 1], c = hull[r];
    	while (1)
    	{
    		vector<int> cur;
    		for (int i = 1; i <= n; i++)
    		{
    			if (i != a && i != b && i != c && inside(a, b, c, i))
    			{
    				cur.push_back(i);
    			}
    		}
    		if (cur.empty())
    		{
    			cout << "! 3 " << a << " " << b << " " << c << endl;
    			break;
    		}
    		if (rand() % 2)
    		{
    			swap(a, b);
    		}
    		if (rand() % 2)
    		{
    			swap(b, c);
    		}
    		if (rand() % 2)
    		{
    			swap(a, c);
    		}
    		int x = cur[rand() % cur.size()];
    		if (ask(a, b, x))
    		{
    			c = x;
    		}
    		else if (ask(b, c, x))
    		{
    			a = x;
    		}
    		else
    		{
    			b = x;
    		}
    	}
    }
}
