#include <iostream>
#include <vector>
#include <algorithm>
#include <map>

using namespace std;

typedef long long ll;

struct Point {
    ll x, y;
    int ind;

    Point(ll x, ll y) :
            x(x),
            y(y) {}

    Point() :
            x(0),
            y(0) {}
};

inline bool cmpxy(const Point &a, const Point &b) {
    return a.x < b.x || (a.x == b.x && a.y < b.y);
}

inline istream &operator>>(istream &in, Point &a) {
    in >> a.x >> a.y;
    return in;
}

struct Vector {
    ll x, y;

    Vector(ll x, ll y) :
            x(x),
            y(y) {}

    Vector(const Point &a, const Point &b) :
            x(b.x - a.x),
            y(b.y - a.y) {}

    ll operator*(const Vector &a) const {
        return x * a.y - y * a.x;
    }
};

inline bool cw(const Point &a, const Point &b, const Point &c) {
    Vector ab(a, b), ac(a, c);
    return ab * ac < 0;
}

inline bool ccw(const Point &a, const Point &b, const Point &c) {
    Vector ab(a, b), ac(a, c);
    return ab * ac > 0;
}

vector<Point> pts;
vector<bool> block;
map<int, int> indInSort;

inline bool ask(vector<Point> &a, bool isAns = false) {
    if (isAns) {
        cout << "! ";
    } else {
        cout << "? ";
    }
    cout << a.size() << " ";
    for (int i = 0; i < a.size(); ++i) {
        cout << a[i].ind + 1 << " ";
    }
    cout << endl;
    string ans;
    if (!isAns) {
        cin >> ans;
    }
    return ans == "Yes";
}

inline bool ch() {
    vector<Point> up, down;
    int st = 0;
    while (block[pts[st].ind]) {
        ++st;
    }
    up.push_back(pts[st]);
    down.push_back(pts[st]);
    int lt = pts.size() - 1;
    while (block[pts[lt].ind]) {
        --lt;
    }
    for (int i = st + 1; i <= lt; ++i) {
        if (block[pts[i].ind]) {
            continue;
        }
        if (!ccw(pts[st], pts[i], pts[lt])) {
            while (up.size() > 1 && ccw(up[up.size() - 2], up[up.size() - 1], pts[i])) {
                up.pop_back();
            }
            up.push_back(pts[i]);
        }
        if (!cw(pts[st], pts[i], pts[lt])) {
            while (down.size() > 1 && cw(down[down.size() - 2], down[down.size() - 1], pts[i])) {
                down.pop_back();
            }
            down.push_back(pts[i]);
        }
    }
    reverse(down.begin(), down.end());
    for (int i = 1; i < down.size() - 1; ++i) {
        up.push_back(down[i]);
    }
    return ask(up);
}

Point base;

inline bool operator==(const Point &a, const Point &b) {
    return a.x == b.x && a.y == b.y;
}

inline bool cmp(const Point &a, const Point &b) {
    if (a == base) {
        return true;
    }
    if (b == base) {
        return false;
    }
    return cw(base, a, b);
}

inline int binSearch(int ind) {
    base = pts[indInSort[ind]];
    vector<Point> pts2 = pts;
    sort(pts2.begin(), pts2.end(), &cmp);
    int l = 0, r = pts.size() - 2;
    while (l < r - 1) {
        int m = (l + r) / 2;
        for (int i = 1; i <= m; ++i) {
            block[pts2[i].ind] = true;
        }
        if (ch()) {
            l = m;
        } else {
            r = m;
        }
        for (int i = 1; i <= m; ++i) {
            block[pts2[i].ind] = false;
        }
    }
    for (int i = 1; i <= l; ++i) {
        block[pts2[i].ind] = true;
    }
    vector<Point> old = pts;
    pts.clear();
    for (int i = 0; i < old.size(); ++i) {
        if (!block[old[i].ind]) {
            pts.push_back(old[i]);
        }
    }
    return pts2[l + 1].ind;
}

int main() {
    ios_base::sync_with_stdio(0);
    cin.tie(NULL);
    cout.tie(NULL);
    int n;
    cin >> n;
    vector<Point> b(n);
    block.resize(n, false);
    for (int i = 0; i < n; ++i) {
        cin >> b[i];
        b[i].ind = i;
    }
    sort(b.begin(), b.end(), &cmpxy);
    int q;
    cin >> q;
    for (int t = 0; t < q; ++t) {
        block.assign(n, false);
        pts = b;
        int ind = pts[0].ind;
        while (pts.size() != 3) {
            for (int i = 0; i < n; ++i) {
                indInSort[pts[i].ind] = i;
            }
            ind = binSearch(ind);
        }
        ask(pts, true);
    }
    return 0;
}
/*
6
3 0
0 2
2 4
4 2
5 5
6 1
4
*/
