#include <iostream>
#include <vector>
#include <string>
#include <set>
#include <map>
#include <cmath>
#include <algorithm>

using namespace std;
const int MAXN = 1e3 * 5 + 100;
const long double EPS = 1e-7;
const long double PI = asin(1.0) * 2;

struct Point{
    long long x;
    long long y;
};

struct Vector{
    long long x;
    long long y;

    Vector(Point& a, Point& b){
        x = b.x - a.x;
        y = b.y - a.y;
    }

    long long cross_product(Vector b){
        return x * b.y - y * b.x;
    }
};

struct VectorLD{
    long double x;
    long double y;

    VectorLD(Point& a, Point& b){
        x = b.x - a.x;
        y = b.y - a.y;
    }

    long double angle(VectorLD& b){
        return atan2(x * b.y - y * b.x, x * b.x + y * b.y);
    }
};

Point mas[MAXN];
pair<Point, int> mas2[MAXN];

struct Triangle{
    int a;
    int b;
    int c;

    bool in_tr(Point& p){
        long double angle = 0;
        VectorLD va = VectorLD(p, mas[a]);
        VectorLD vb = VectorLD(p, mas[b]);
        VectorLD vc = VectorLD(p, mas[c]);

        angle += va.angle(vb);
        angle += vb.angle(vc);
        angle += vc.angle(va);

        angle = abs(angle);

        if (abs(angle - PI * 2) < EPS || abs(va.angle(vb)) - PI < EPS || abs(vb.angle(vc)) - PI < EPS || abs(vc.angle(va)) - PI < EPS){
            return true;
        } else {
            return false;
        }
    }
};

int min_p = 0;

bool cmt(pair<Point, int>& a, pair<Point, int>& b){
    return (a.first.x * b.first.y - a.first.y * b.first.x) > 0;
}

vector<Triangle> trian;

int main(){
    ios::sync_with_stdio(false);
    cin.tie(0);

    int n;
    cin >> n;

    for (int i = 0; i < n; ++i){
        cin >> mas[i].x >> mas[i].y;
        mas2[i].first = mas[i];
        mas2[i].second = i;
    }

    for (int i = 0; i < n; ++i){
        if (mas[i].y < mas[min_p].y){
            min_p = i;
        } else if (mas[i].y == mas[min_p].y && mas[i].x < mas[min_p].x){
            min_p = i;
        }
    }

    swap(mas2[0], mas2[min_p]);
    sort(mas2 + 1, mas2 + n, cmt);

    for (int i = 0; i < n; ++i){
        mas[i] = mas2[i].first;
    }

    vector<int> stck;
    stck.push_back(0);

    for (int i = 1; i < n; ++i){
        while (stck.size() > 1 && Vector(mas[stck[stck.size() - 2]], mas[stck[stck.size() - 1]]).cross_product(Vector(mas[stck[stck.size() - 2]], mas[i])) < 0){
            stck.pop_back();
        }

        stck.push_back(i);
    }


    set<int> st;

    for (int i = 0; i < n; ++i){
        st.insert(i);
    }

    for (int i = 0; i < stck.size(); ++i){
        st.erase(stck[i]);
    }

    for (int i = 1; i < stck.size() - 1; ++i){
        trian.push_back(Triangle({0, stck[i], stck[i + 1]}));
    }



    for (auto e : st){
        int j = 0;
        while (j < trian.size()){
            if (trian[j].in_tr(mas[e])){
                int a = trian[j].a;
                int b = trian[j].b;
                int c = trian[j].c;

                swap(trian[trian.size() - 1], trian[j]);

                trian.pop_back();
                trian.push_back({e, a, b});
                trian.push_back({e, b, c});
                trian.push_back({e, a, c});
                break;
            }
            ++j;
        }
    }

    int q;
    cin >> q;

    for (int k = 0; k < q; ++k){
        for (int i = 0; i < trian.size(); ++i){
            cout << "? " << 3 << ' ' << mas2[trian[i].a].second + 1 << ' ' << mas2[trian[i].b].second + 1 << ' ' << mas2[trian[i].c].second + 1 << endl;

            string an;
            cin >> an;

            if (an == "Yes"){
                cout << "! " << 3 << ' ' << mas2[trian[i].a].second + 1 << ' ' << mas2[trian[i].b].second + 1 << ' ' << mas2[trian[i].c].second + 1 << endl;
                break;
            }
        }
    }

    return 0;
}
