#include <algorithm>
#include <fstream>
#include <cmath>
#include <random>
#include <cassert>
#include <ctime>
#include <set>
#include <map>
#include <bitset>
#include <string>
#include <vector>
#include <utility>

using namespace std;

template<class t> inline void chkmin(t& a, t b) { if (a > b) a = b; }
template<class t> inline void chkmax(t& a, t b) { if (a < b) a = b; }

typedef long long ll;
typedef long double ld;
typedef pair<ll, ll> seg;

const string FILENAME = "internship";
ifstream cin(FILENAME + ".in");
ofstream cout(FILENAME + ".out");

const ll INF = 1e18;

ll MyRand() {
	return (rand() << 16) ^ rand();
}

struct Treap {
	ll y;
	int beg;
	int len;
	ll val;
	ll max;
	ll size;
	ll sum;
	Treap* left;
	Treap* right;

	Treap(ll val, int beg, int len) : beg(beg), len(len), y(MyRand()), val(val), max(val), size(1), left(0), right(0), sum(val) {}
};

ll max(Treap* t) { return t ? t->max : -INF; }
ll size(Treap* t) { return t ? t->size : 0; }
ll sum(Treap* t) { return t ? t->sum : 0; }

void Update(Treap* t) {
	t->max = max(t->val, max(max(t->left), max(t->right)));
	t->size = 1 + size(t->left) + size(t->right);
	t->sum = t->len + sum(t->left) + sum(t->right);
}

Treap* Merge(Treap* a, Treap* b) {
	if (a == 0) return b;
	if (b == 0) return a;
	if (a->y < b->y) {
		a->right = Merge(a->right, b);
		Update(a);
		return a;
	} else {
		b->left = Merge(a, b->left);
		Update(b);
		return b;
	}
}

typedef pair<Treap*, Treap*> treap_pair;

treap_pair Split(Treap* t, int k) {
	if (t == 0) return treap_pair(0, 0);
	treap_pair p(0, 0);
	if (size(t->left) < k) {
		p = Split(t->right, k - 1 - size(t->left));
		t->right = p.first;
		p.first = t;
	} else {
		p = Split(t->left, k);
		t->left = p.second;
		p.second = t;
	}
	Update(t);
	return p;
}

treap_pair SplitSum(Treap* t, ll k) {
	if (t == 0) return treap_pair(0, 0);
	//if (t->sum <= k) return treap_pair(t, 0);
	//if (t->left == 0 && t->right == 0 && k < t->len) return treap_pair(0, t);
	treap_pair p(0, 0);
	if (sum(t->left) + t->len < k) {
		p = Split(t->right, k - t->len - sum(t->left));
		t->right = p.first;
		p.first = t;
	} else {
		p = Split(t->left, k);
		t->left = p.second;
		p.second = t;
	}
	Update(t);
	return p;
}

const int MAXN = 10001;

int in[2][MAXN];
int type[2][MAXN];
ll pref[2][MAXN];
bitset<MAXN> used;

int main() {
	int n, m;
	cin >> n >> m;
	for (int q = 0; q < 2; ++q) {
		int cn = q ? m : n;
		for (int i = 0; i < cn; ++i) {
			cin >> type[q][i];
			in[q][type[q][i]] = i;
		}
		for (int i = 0; i < cn; ++i) {
			cin >> pref[q][i + 1];
			pref[q][i + 1] += pref[q][i];
		}
	}
	ll res = pref[1][m];
	seg ans1(-1, -1), ans2(0, m - 1);
	if (res < pref[0][n]) {
		res = pref[0][n];
		ans1 = seg(0, n - 1);
		ans2 = seg(-1, -1);
	}
	ll resqqq = res;
	seg ans1qqq = ans1, ans2qqq = ans2;
	for (int i1 = 0; i1 < n; ++i1) {
		for (int j = 0; j <= m + n; ++j)
			used[j] = 0;
		//Treap* t = new Treap(pref[1][m], 0, m);
		for (int i2 = i1; i2 < n; ++i2) {
			used[type[0][i2]] = true;
			for (int j1 = 0; j1 < m; ++j1) {
				for (int j2 = j1; j2 < m; ++j2) {
					if (used[j2]) break;
					ll cres = pref[0][i2 + 1] - pref[0][i1] + pref[1][j2 + 1] - pref[1][j1];
					if (resqqq < cres) {
						resqqq = cres;
						ans1qqq = seg(i1, i2);
						ans2qqq = seg(j1, j2);
					}
				}
				//if (j1 != -1 && !used[type[1][j1]]) continue;
				//int j2 = j1 + 1;
				//while (j2 < m && !used[type[1][j2]]) ++j2;
				//ll cres = pref[0][i2 + 1] - pref[0][i1] + pref[1][j2] - pref[1][j1 + 1];
				//if (res < cres) {
				//	res = cres;
				//	ans1 = seg(i1, i2);
				//	ans2 = seg(j1 + 1, j2 - 1);
				//}
			}
			//treap_pair p = SplitSum(t, j);
			//Treap* a = p.first;
			//p = Split(p.second, 1);
			//Treap* b = p.first;
			//int j1 = b->beg;
			//int j2 = j1 + b->len;
			//Treap* d = p.second;
			//Treap* c = new Treap(pref[1][j2] - pref[1][j], j, j2 - j);
			//b->len = j - j1;
			//b->val = pref[1][j] - pref[1][j1];
			//Update(b);
			//t = Merge(Merge(a, b), Merge(c, d));
			//chkmax(res, pref[0][i2 + 1] - pref[0][i1] + t->max);
		}
	}
	assert(res == resqqq);
	//cout << res << '\n';
	//cout << ans1.first + 1 << ' ' << ans1.second + 1 << '\n';
	//cout << ans2.first + 1 << ' ' << ans2.second + 1 << '\n';
	cout << resqqq << '\n';
	cout << ans1qqq.first + 1 << ' ' << ans1qqq.second + 1 << '\n';
	cout << ans2qqq.first + 1 << ' ' << ans2qqq.second + 1 << '\n';
}