#include <iostream>
#include <algorithm>
#include <vector>
#include <stdlib.h>
#include <string>
#include <time.h>
#include <cmath>

using namespace std;

typedef long long ll;
typedef long double ld;

#define rand_int ((rand() << 15) | rand())

bool check(vector<int> v)
{
    cout << "? " << v.size() << " ";
    for (int i : v)
        cout << i + 1 << " ";
    cout << endl;
    string f;
    cin >> f;
    return (f == "Yes");
}

void write(vector<int> v)
{
    cout << "! " << v.size() << " ";
    for (int i : v)
        cout << i + 1 << " ";
    cout << endl;
}

struct point
{
    int x, y;
    int num;

    point()
    {
        x = y = num = 0;
    }

    point(int a, int b, int i)
    {
        x = a;
        y = b;
        num = i;
    }

    bool operator==(const point &t) const
    {
        return (x == t.x && y == t.y);
    }
};

struct vect
{
    int x, y;

    vect()
    {
        x = y = 0;
    }

    vect(int a, int b)
    {
        x = a;
        y = b;
    }

    vect(point p1, point p2)
    {
        x = p2.x - p1.x;
        y = p2.y - p1.y;
    }

    ll operator*(const vect &t) const
    {
        return ((ll)x * (ll)t.x + (ll)y * (ll)t.y);
    }

    ll operator^(const vect &t) const
    {
        return ((ll)x * (ll)t.y - (ll)y * (ll)t.x);
    }
};

struct polygon
{
    vector<point> v;

    polygon()
    {
        v = {};
    }

    polygon(vector<point> p)
    {
        v = p;
    }
};

point now;

bool cmp(point a, point b)
{
    vect v1 = vect(now, a);
    vect v2 = vect(now, b);
    return ((v1 ^ v2) > 0);
}

polygon convex_hull(polygon p)
{
    point u = p.v[0];
    int n = p.v.size();
    for (int i = 0; i < n; i++)
    {
        if (p.v[i].y < u.y)
            u = p.v[i];
        else if (p.v[i].y == u.y && p.v[i].x < u.x)
            u = p.v[i];
    }
    now = u;
    vector<point> nw;
    for (int i = 0; i < n; i++)
        if (p.v[i].num != u.num)
            nw.push_back(p.v[i]);
    sort(nw.begin(), nw.end(), cmp);
    vector<point> ans;
    ans.push_back(u);
    ans.push_back(nw[0]);
    for (int i = 2; i < n; i++)
    {
        while (ans.size() > 1)
        {
            point p2 = ans[ans.size() - 1];
            point p1 = ans[ans.size() - 2];
            vect v1 = vect(p1, p2);
            vect v2 = vect(p2, nw[i - 1]);
            if ((v1 ^ v2) < 0)
            {
                ans.pop_back();
                continue;
            }
            break;
        }
        ans.push_back(nw[i - 1]);
    }
    return polygon(ans);
}

ld heig(point a, point b, point c)
{
    vect u = vect(a, b);
    ld t = u * vect(a, c);
    if (t < 0)
        return sqrt(vect(a, c) * vect(a, c));
    ld pr = t / sqrt(u * u);
    return sqrt((vect(a, c) * vect(a, c)) - pr * pr);
}

int n;
vector<point> v;
polygon st;

ll sqr(vector<point> &r)
{
    return abs(vect(r[0], r[1]) ^ vect(r[0], r[2]));
    ll ans = 0;
    for (int i = 0; i < (int)r.size() - 1; i++)
        ans += (vect(point(0, 0, 0), r[i]) ^ vect(point(0, 0, 0), r[i + 1]));
    ans += (vect(point(0, 0, 0), r[n - 1]) ^ vect(point(0, 0, 0), r[0]));
    return abs(ans);
}

bool isin(vector<point> &p, point x, ll s)
{
    ll now = 0;
    vector<point> u = {p[0], p[1], x};
    now += sqr(u);
    u = {p[1], p[2], x};
    now += sqr(u);
    u = {p[2], p[0], x};
    now += sqr(u);
    return (now == s);
}

bool test(vector<point> &p)
{
    vector<int> ans;
    for (int i = 0; i < p.size(); i++)
        ans.push_back(p[i].num);
    return check(ans);
}

void answer()
{
    int ls = 1;
    int rs = (int)st.v.size() - 1;
    while (rs - ls > 1)
    {
        int h = (ls + rs) >> 1;
        vector<int> now;
        for (int i = 0; i <= h; i++)
            now.push_back(st.v[i].num);
        if (check(now))
            rs = h;
        else
            ls = h;
    }
    vector<point> r;
    r = {st.v[0], st.v[ls], st.v[ls + 1]};
    ll s = sqr(r);
    vector<int> in;
    for (int i = 0; i < n; i++)
    {
        if (i != r[0].num && i != r[1].num && i != r[2].num)
        {
            if (isin(r, v[i], s))
                in.push_back(i);
        }
    }
    while (true)
    {
        if (in.empty())
        {
            vector<int> ans;
            for (int i = 0; i < r.size(); i++)
                ans.push_back(r[i].num);
            write(ans);
            return;
        }
        vector<pair<ld, int> > xt;
        for (int i : in)
            xt.push_back(make_pair(heig(r[0], r[1], v[i]), i));
        nth_element(xt.begin(), xt.end(), xt.begin() + (xt.size() / 2));
        point x = v[xt[xt.size() / 2].second];
        vector<point> v1 = {r[0], r[1], x};
        vector<point> v2 = {r[1], r[2], x};
        vector<point> v3 = {r[2], r[0], x};
        if (test(v1))
            r = v1;
        else if (test(v2))
            r = v2;
        else
            r = v3;
        ll s = sqr(r);
        vector<int> d;
        for (int i : in)
        {
            if (i != r[0].num && i != r[1].num && i != r[2].num)
            {
                if (isin(r, v[i], s))
                    d.push_back(i);
            }
        }
        in = d;
    }
}

int main()
{
    ios_base::sync_with_stdio(0);
    cin >> n;
    for (int i = 0; i < n; i++)
    {
        int x, y;
        cin >> x >> y;
        v.push_back(point(x, y, i));
    }
    st = convex_hull(polygon(v));
    int q;
    cin >> q;
    for (int z = 0; z < q; z++)
        answer();
    return 0;
}

