#include <iostream>
#include <cstdio>
#include <vector>
#include <memory.h>
#include <cmath>
#include <algorithm>

typedef long double ld;

using namespace std;

const ld pi = 3.141592653589793238462643, EPS = 1e-14;

struct Point {
    ld x, y;
    Point() {
        x = 0;
        y = 0;
    }
    Point (ld a, ld b) {
        x = a;
        y = b;
    }
};

struct Vector {
    ld x, y;
    Vector() {
        x = 0, y = 0;
    }
    Vector (ld a, ld b) {
        x = a, y = b;
    }
    Vector (Point a, Point b) {
        x = b.x - a.x, y = b.y - a.y;
    }
};

ld sp(Vector a, Vector b) {
    return a.x * b.x + a.y * b.y;
}

ld vp(Vector a, Vector b) {
    return b.x * a.y - a.x * b.y;
}

ld polar(Vector a) {
    Vector b(1, 0);
    ld angle = atan2(vp(a, b), sp(a, b));
    if (angle < -EPS) {
        angle += 2 * pi;
    }
    return angle;
}

bool cmp(pair <Point, int> a, pair <Point, int> b) {
    Vector a1(Point(0, 0), a.first), b1(Point(0, 0), b.first);
    ld an1 = polar(a1), an2 = polar(b1);
    return an2 - an1 > EPS;
}

const int maxn = 65;

bool used[maxn];

vector <int> Graham(vector <pair <Point, int>> v) {
    int t = 0;
    while (used[t]) {
        ++t;
    }
    Point P = v[t].first;
    int n = v.size();
    for (int i = 0; i < n; ++i) {
        if (used[i]) {
            continue;
        }
        if (P.y - v[i].first.y > EPS) {
            P = v[i].first;
        } else if (fabs(v[i].first.y - P.y) < EPS && v[i].first.x - P.x > EPS) {
            P = v[i].first;
        }
    }
    //cout << P.x << " " << P.y << endl;
    for (int i = 0; i < n; ++i) {
        v[i].first.x -= P.x;
        v[i].first.y -= P.y;
    }
    sort(v.begin(), v.end(), cmp);
    vector <Point> hull;
    vector <int> ans;
    for (int i = 0; i < n; ++i) {
        if (used[v[i].second - 1]) {
            continue;
        }
        //cout << i << " " << v[i].x << " " << v[i].y << endl;
        if (hull.size() < 2) {
            hull.push_back(v[i].first);
            ans.push_back(v[i].second);
        } else {
            Vector a(hull[hull.size() - 2], hull.back()), b(hull.back(), v[i].first);
            while (hull.size() >= 2 && vp(a, b) > -EPS) {
                hull.pop_back();
                ans.pop_back();
                if (hull.size() >= 2) {
                    a = Vector(hull[hull.size() - 2], hull[hull.size() - 1]);
                    b = Vector(hull.back(), v[i].first);
                }
            }
            hull.push_back(v[i].first);
            ans.push_back(v[i].second);
        }
    }
    //cout << hull.size() << endl;
    //for (int i = 0; i < hull.size(); ++i) {
    //    hull[i].x += P.x;
    //    hull[i].y += P.y;
    //}
    return ans;
}

int main()
{
    //freopen("input.txt", "r", stdin);
    //freopen("output.txt", "w", stdout);
    ios_base::sync_with_stdio(0);
    int n;
    cin >> n;
    vector <pair <Point, int>> v;
    for (int i = 0; i < n; ++i) {
        Point P;
        cin >> P.x >> P.y;
        v.push_back({P, i + 1});
    }
    //vector <int> hull = Graham(v);
    //for (int i = 0; i < hull.size(); ++i) {
    //    cout << hull[i] << " ";
    //}
    int q;
    cin >> q;
    for (int i = 0; i < q; ++i) {
        vector <int> gg = Graham(v);
        memset(used, 0, sizeof(used));
        for (int t = 0; t < n; ++t) {
            for (int j = 0; j < gg.size(); ++j) {
                if (!used[gg[j] - 1]) {
                    used[gg[j] - 1] = true;
                    vector <int> hull = Graham(v);
                    int sz = hull.size();
                    if (sz < 3) {
                        used[gg[j] - 1] = false;
                        continue;
                    }
                    cout << "? " << " " << sz << " ";
                    for (int k = 0; k < sz; ++k) {
                        cout << hull[k] << " ";
                    }
                    cout << endl;
                    string res;
                    cin >> res;
                    if (res == "No") {
                        used[gg[j] - 1] = false;
                    } else {
                        gg = hull;
                        break;
                    }
                }
            }
        }
        vector <int> hull = Graham(v);
        cout << "! ";
        cout << hull.size() << " ";
        for (int j = 0; j < hull.size(); ++j) {
            cout << hull[j] << " ";
        }
        cout << endl;
    }
    return 0;
}
