#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

struct point {
    long long x, y;
    int id;

    point(long long x, long long y, int id): x(x), y(y), id(id) {}
    point(long long x, long long y): x(x), y(y), id(0) {}
    point(): x(0), y(0), id(0) {}

    point(const point &a, const point &b) {
        x = b.x - a.x;
        y = b.y - a.y;
    }
};

point operator-(const point &a, const point &b) {
    return point(a.x - b.x, a.y - b.y);
}

long long operator%(const point &a, const point &b) {
    return a.x * b.y - b.x * a.y;
}

struct triangle {
    point a, b, c;

    int ask() {
        cout << "? 3 " << a.id << ' ' << b.id << ' ' << c.id << endl;
        string ans;
        cin >> ans;
        if (ans == "Yes") {
            cout << "! 3 " << a.id << ' ' << b.id << ' ' << c.id << endl;
            return 1;
        }
        return 0;
    }
};

typedef vector<point> poly;

point good(0, 0);
bool compare(const point &a, const point &b) {
    point p1(good, a), p2(good, b);
    if (p1.x == p1.y && p1.y == 0)
        return false;
    if (p2.x == p2.y && p2.y == 0)
        return true;
    return p1 % p2 > 0;
}


vector<triangle> algo(vector<point> points) {
    int n = points.size();
    good = points[0];
    for (int i = 1; i < n; ++i) {
        if (points[i].x < good.x || (points[i].x == good.x && points[i].y < good.y))
            good = points[i];
        sort(points.begin(), points.end(), compare);
    }
    vector<triangle> ans;
    poly result;
    result.push_back(points[0]);
    int size = 1;
    for (int i = 1; i < n - 1; i++) {
        ans.push_back({good, points[i - 1], points[i]});
        while (size >= 2) {
            point v1(result[size-2], result[size-1]),
                    v2(result[size-1], points[i]);
            if (v1 % v2 > 0)
                break;
            ans.push_back({result[size-2], result[size-1], points[i]});
            result.pop_back();
            --size;
        }
        result.push_back(points[i]);
        ++size;
    }
    return ans;
}

int main() {
    int n;
    cin >> n;
    vector<point> p(n);
    for (int i = 0; i < n; i++) {
        cin >> p[i].x >> p[i].y;
        p[i].id = i + 1;
    }

    vector<triangle> lst = algo(p);

    int q;
    string ans;
    cin >> q;
    while (q-- > 0) {
        bool found = false;
        for (auto& tr: lst)
            if (tr.ask()) {
                found = true;
                break;
            }
        if (!found) {
            cout << "! " << n << ' ';
            for (int i = 1; i <= n; i++)
                cout << i << ' ';
            cout << endl;
        }
    }
    return 0;
}