#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <stack>

using namespace std;

typedef long long ll;
#define POHER 0
#define XOR 1
#define NOT_XOR 2
#define CANT 3

ll n, q;
ll mas[1000000];

int get_bit(ll a, int bit) {
	return (a >> bit) & 1;
}

void set_bit(ll &a, int bit, int change) {
	if (change == 1) {
		a |= (1 << bit);
	}
	else
		a &= ~(1 << bit);
}

struct DO {
	int memory[4 * 1000000];
	int build(int v, int vl, int vr, int bit_level) {
		if (vr == vl + 1) {
			memory[v] = get_bit(mas[vl], bit_level);
			return memory[v];
		}
		int m = (vl + vr) / 2;
		memory[v] = build(v * 2, vl, m, bit_level) + build(v * 2 + 1, m, vr, bit_level);
		return memory[v];
	}

	int sum(int v, int vl, int vr, int l, int r) {
		if (vl >= l && vr <= r)
			return memory[v];
		if (vl >= r || vr <= l)
			return 0;
		int m = (vl + vr) / 2;
		return sum(2 * v, vl, m, l, r) + sum(2 * v + 1, m, vr, l, r);
	}

	void update(int v, int vl, int vr, int x, int delta) {
		if (x >= vl && x < vr) {
			memory[v] += delta;
			if (vr != vl + 1) {
				int m = (vl + vr) / 2;
				update(2 * v, vl, m, x, delta);
				update(2 * v + 1, m, vr, x, delta);
			}
		}
	}
};

struct Status {
	bool xor = false, not_xor = false;
};

DO bit_levels[31];
Status stat_for_level[31];

int find_first_one(int bit_level, int lc, int rc, int sum) {
	int l = lc;
	int r = rc;
	while (r - l > 1) {
		int m = (l + r) / 2;
		if (bit_levels[bit_level].sum(1, 0, n, m, rc) == sum) {
			l = m;
		}
		else
			r = m;
	}
	return l;
}

int find_last_one(int bit_level, int lc, int rc, int sum) {
	int l = lc;
	int r = rc;
	while (r - l > 1) {
		int m = (l + r) / 2;
		if (bit_levels[bit_level].sum(1, 0, n, lc, m ) == sum) {
			r = m;
		}
		else
			l = m;
	}
	return l;
}

int get_status(int bit_level, int l, int r, int& splitter) {
	int sum = bit_levels[bit_level].sum(1, 0, n, l, r);
	if (sum == r - l || sum == 0) {
		return POHER;
	}
	else if (get_bit(mas[l], bit_level) == 0) {
		int first = find_first_one(bit_level, l, r, sum);
		splitter = first;
		if (r - first == sum) {
			return NOT_XOR;
			
		}
		else
			return CANT;
	}
	else {
		int last = find_last_one(bit_level, l, r, sum);
		splitter = last + 1;
		if (last - l + 1 == sum) {
			return XOR;
		}
		else
			return CANT;
	}
}

bool init_status(int bit_level, int l, int r) {
	if (bit_level < 0 || l >= r)
		return true;
	int splitter;
	if (bit_level == 2)
		cout << "";
	switch (get_status(bit_level, l, r, splitter)) {
	case XOR: {
		stat_for_level[bit_level].xor = true;
		if (!init_status(bit_level - 1, l, splitter))
			return false;
		if (!init_status(bit_level - 1, splitter, r))
			return false;
		break;
	}
	case NOT_XOR: {
		stat_for_level[bit_level].not_xor = true;
		if (!init_status(bit_level - 1, l, splitter))
			return false;
		if (!init_status(bit_level - 1, splitter, r))
			return false;
		break;
	}
	case POHER: {
		if (!init_status(bit_level - 1, l, r))
			return false;
		break;
	}
	case CANT: {
		return false;
		break;
	}
	}
	return true;
}

ll get_ans() {
	ll ans = 0;
	for (int bit_level = 0; bit_level < 31; bit_level++) {
		if (stat_for_level[bit_level].xor && stat_for_level[bit_level].not_xor)
			return -1;
		if (stat_for_level[bit_level].xor)
			set_bit(ans, bit_level, 1);
	}
	return ans;
}

void print_ans() {
	for (int bit_level = 0; bit_level < 31; bit_level++) {
		stat_for_level[bit_level].xor = false;
		stat_for_level[bit_level].not_xor = false;
	}
	if (!init_status(30, 0, n)) {
		cout << "-1" << endl;
		return;
		}
	ll ans = get_ans();
	cout << ans << endl;
}

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);
	cout.tie(0);
	cin >> n;
	for (ll i = 0; i < n; i++) {
		cin >> mas[i];
	}
	for (int bit_level = 0; bit_level < 31; bit_level++) {
		bit_levels[bit_level].build(1, 0, n, bit_level);
	}
	
	print_ans();
	cin >> q;
	for (ll i = 0; i < q; i++) {
		ll p, v;
		cin >> p >> v;
		p--;
		for (int bit_level = 0; bit_level < 31; bit_level++) {
			int delta = get_bit(v, bit_level) - get_bit(mas[p], bit_level);
			if (delta != 0)
				bit_levels[bit_level].update(1, 0, n, p, delta);
			set_bit(mas[p], bit_level, get_bit(v, bit_level));
		}
		print_ans();
	}
	return 0;
}